import {Directive} from '@angular/core';
import {AbstractControl, AsyncValidator, NG_ASYNC_VALIDATORS, ValidationErrors} from '@angular/forms';
import {Observable, of} from 'rxjs';

import Swal from "sweetalert2";

import {Currencies} from './currencies.component'; // For simulation of forbidden ISO codes only, otherwise bad coupling...

interface Iso_code_checking { // Interface as contract...
    _is_forbidden: (iso_code: string) => boolean;
}

@Directive({
    // Angular locates each element in the template that has an attribute named 'forbidden_ISO_code'...
    selector: '[forbidden_ISO_code]', // Brackets ([]) make it an attribute selector...
    // Validators have to be registered so Angular is aware of them and can use them during binding:
    providers: [{provide: NG_ASYNC_VALIDATORS, useExisting: My_asynchronous_validator, multi: true}]
})
export class My_asynchronous_validator implements AsyncValidator, Iso_code_checking {
    _is_forbidden(iso_code: string): boolean {
        let timerInterval
        Swal.fire({
            title: 'Asynchronous validation in progress...',
            html: 'Close in <b></b> milliseconds.',
            timer: 1500,
            timerProgressBar: true,
            onBeforeOpen: () => {
                Swal.showLoading()
                timerInterval = window.setInterval(() => {
                    const content: HTMLElement = Swal.getContent();
                    if (content)
                        content.querySelector('b')!.textContent = Swal.getTimerLeft().toString();
                }, 100)
            },
            onClose: () => {
                window.clearInterval(timerInterval)
            }
        }).then((result) => {
            /* Read more about handling dismissals below */
            if (result.dismiss === Swal.DismissReason.timer)
                window.console.log('Asynchronous validation completes...')
        });
        return Currencies.Currencies.find(currency => currency.iso_code.toString() === iso_code) ? true : false;
    }

    validate(ac: AbstractControl): Observable<ValidationErrors | null> {
// From the doc.:
//         type ValidationErrors = {
//             [key: string]: any;
//         };
        return this._is_forbidden(ac.value) ? of({my_custom_error: ac.value}) : of(null);
    }
}
