// 'currencies-controller.component.ts' file
// tslint:disable
import {Component, OnInit} from '@angular/core';
import {
    AbstractControl,
    AsyncValidatorFn,
    /* FormBuilder, */
    FormControl,
    FormGroup,
    ValidationErrors,
    ValidatorFn,
    Validators
} from '@angular/forms';

import Swal from "sweetalert2";

import {Currencies} from './currencies.component';
import {Change_route_service} from "./change-route-service";

@Component({
    selector: 'currencies-controller',
    styleUrls: ['./currencies-controller.component.css'],
    templateUrl: './currencies-controller.component.html'
})
export class Currencies_controller implements OnInit {
    static My_custom_validator: ValidatorFn = (new_currency: FormGroup): ValidationErrors | null => {
        const common_symbol = new_currency.get('common_symbol');
        const iso_symbol = new_currency.get('iso_symbol');
        return common_symbol && iso_symbol && common_symbol.value === iso_symbol.value ? {'my_custom_error': true} : null; // 'null' -> no error...
    };

    static My_asynchronous_custom_validator: AsyncValidatorFn = (ac: AbstractControl): Promise<ValidationErrors | null> /*| Observable<ValidationErrors | null>*/ => {
        let timerInterval
        Swal.fire({
            title: 'Asynchronous validation in progress...',
            html: 'Close in <b></b> milliseconds.',
            timer: 2500,
            timerProgressBar: true,
            onBeforeOpen: () => {
                Swal.showLoading()
                timerInterval = window.setInterval(() => {
                    const content: HTMLElement = Swal.getContent();
                    if (content)
                        content.querySelector('b')!.textContent = Swal.getTimerLeft().toString();
                }, 100)
            },
            onClose: () => {
                window.clearInterval(timerInterval)
            }
        }).then((result) => {
            /* Read more about handling dismissals below */
            if (result.dismiss === Swal.DismissReason.timer)
                window.console.log('Asynchronous validation completes...')
        });
        return Currencies.Currencies.find(currency => currency.iso_code.toString() === ac.value) ?
            Promise.resolve({my_custom_error: ac.value}) :
            Promise.resolve(null); // 'null' -> no error...
    };
    public readonly default = Currencies.Currencies[Currencies.Dollar];
    public readonly default_date = new Date(2020, 4, 11, 0, 0, 0, 0);
    public two_way_binding: string = this.default.common_name;

    private readonly _common_name: FormControl = new FormControl;
    get common_name(): FormControl {
        return this._common_name;
    }

    private readonly _common_symbol: FormControl = new FormControl('', [Validators.required]);
    get common_symbol(): FormControl {
        return this._common_symbol;
    }

    private readonly _description: FormControl = new FormControl('');
    get description(): FormControl {
        return this._description;
    }

    private readonly _iso_code: FormControl = new FormControl('',
        {
            validators: [Validators.required, Validators.pattern('^(?!000)(\\d{3}$)')],
            //asyncValidators: [Currencies_controller.My_asynchronous_custom_validator] // 'AsyncValidatorFn' objects here...
        });
    get iso_code(): FormControl {
        return this._iso_code;
    }

    private readonly _iso_symbol: FormControl = new FormControl('', [Validators.required,
        Validators.pattern(new RegExp('^[A-Z]{3}$'))]);
    get iso_symbol(): FormControl {
        return this._iso_symbol;
    }

    private readonly _substitution_date: FormControl = new FormControl('');
    get substitution_date(): FormControl {
        return this._substitution_date;
    }

    public readonly new_currency = new FormGroup({
        common_name: this.common_name,
        common_symbol: this.common_symbol,
        description: this.description,
        iso_code: this.iso_code,
        iso_symbol: this.iso_symbol,
        substitution_date: this.substitution_date
    }, {validators: Currencies_controller.My_custom_validator}); // <-- Here's how the custom validator is passed...

    ngOnInit() {
        this.common_name.valueChanges.subscribe(character => {
            // https://codecraft.tv/courses/angular/forms/reactive-model-form/
            window.console.log(JSON.stringify(character));
        });
    }

    public reset_common_name(): void {
        this.new_currency.patchValue({common_name: ''});
        // Alternative:
        // this._common_name.setValue("");
    }

    constructor(/*private readonly _form_builder: FormBuilder*/
                private readonly _change_route_service: Change_route_service) {
    } // Dependency injection
    public record_currency(): void {
        // if (this.new_currency.valid) { // Test is redundant since 'record_currency' is launched when 'this.new_currency.valid'...
        // window.console.log('Form status: ' + this.new_currency.status + '\nvalue: ' + JSON.stringify(this.new_currency.value));
        /* Add new data in 'Currencies.Currencies'... */
        /* Add new corresponding route in 'Currencies_menu.Navigations'... */
        const next_round = Math.floor(Math.random() * 2); // For the fun: '0' or '1'...
        if (next_round === 0) {
            Swal.fire({
                position: 'top-end',
                icon: 'success',
                title: '(form reset) -> "New" is following navigation...',
                showConfirmButton: false,
                timer: 1500
            });
            this.new_currency.reset(); // Next round...
        } else {
            window.console.assert(next_round === 1);
            Swal.fire({
                position: 'top-end',
                icon: 'success',
                title: '(form quit) -> "Currencies" is following navigation...',
                showConfirmButton: false,
                timer: 1500
            });
            this._change_route_service.send_path('Currencies'); // Next round...
        }
        // }
    }
}
