// 'currencies-information.component.ts' file
// tslint:disable
import {AfterViewInit, Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {Location} from '@angular/common';
import {ActivatedRoute, ParamMap} from "@angular/router";

import {Currencies} from './currencies.component';

@Component({
    selector: 'currencies-information',
    template: `
        <div>
            <h2 #my_h2
                [ngClass]="{'MY_BLUE': true, 'MY_RED': information().includes(default_substitute_common_symbol)}">{{information() | uppercase}}</h2>
            <span class="material-icons">{{"?" | guess_material_icon: this.material_icon}}</span>
            <hr>
            <button (click)="back()">Back (leave from '{{this.location.path()}}')</button>
        </div>
    `,
    styles: ['div{border: 5px dashed purple;}', '.MY_BLUE{color: blue}', '.MY_RED{color: red}'] // https://angular.io/guide/component-styles ('--inline-style')
})
export class Currencies_information implements AfterViewInit, OnInit {
    @ViewChild('my_h2') // Injection...
    private _my_h2: ElementRef; // https://www.techiediaries.com/angular-elementref/
    ngAfterViewInit() {
        window.console.log(this._my_h2.nativeElement.innerText); // 'nativeElement' <=> 'current' in React
    }

    public readonly default_substitute_common_symbol = Currencies.Default_substitute().common_symbol;
    private _information: string = "";

    constructor(readonly location: Location, private readonly _navigation: ActivatedRoute) { // Dependency injection
        // Don't immediately use 'location' and '_navigation'...
    }

    public back(): void {
        this.location.back();
    }

    public information(): string {
        return "Here: " + this._information;
    }

    private _material_icon: string;
    get material_icon(): string {
        return this._material_icon;
    }

    set material_icon(material_icon: string) {
        this._material_icon = material_icon;
    }

    ngOnInit() {
        this._navigation.paramMap.subscribe((parameters: ParamMap) => { // Find 'material_icon' from 'iso_code':
            const currency = Currencies.Currencies.find(currency => currency.iso_code.toString() === parameters.get("iso_code") /* '840' for Dollar, etc. */);
            this.material_icon = currency && currency.hasOwnProperty('material_icon') ? currency.material_icon : "?";
        });
        this._navigation.url.subscribe((x) => { // Any time URL changes, this callback is fired...
            const currency = Currencies.Currencies.find(currency => {
                return "/" + `${currency.iso_code}` === this._navigation['_routerState'].snapshot.url;
            });
            this._information = this._navigation.snapshot.toString() + " - " + currency.common_symbol;
        });
    }
}
