// 'currencies-service.ts' file
// tslint:disable
import {Injectable} from '@angular/core';
import {Subject} from 'rxjs';

import {Currency} from './currencies.component';
// Used for '{providedIn: Currencies_module}'
// import Currencies_module from './currencies.module'; // Problem: circular dependency...

@Injectable({providedIn: 'root'}) // 'root' injector -> Angular creates a single, shared instance...
export class Currencies_service {
    // https://makina-corpus.com/blog/metier/2017/premiers-pas-avec-rxjs-dans-angular
    // To be replaced by 'BehaviorSubject' while 'AsyncSubject' is one-shot only through 'complete':
    private _exchange_rate: Subject<number> = new Subject();
    get exchange_rate(): Subject<number> {
        return this._exchange_rate;
    }

    private _from: Currency;
    private _to: Currency;
    private readonly _request = new XMLHttpRequest();

    constructor() {
        this._request.onreadystatechange = () => { // Get the result in an asynchronous way...
            // window.console.warn(this._request.getAllResponseHeaders());
            if (this._request.readyState === XMLHttpRequest.DONE) {
                if (this._request.getResponseHeader('Content-Type').includes('application/json')) {
                    const result = JSON.parse(this._request.responseText);
                    // window.alert(this._request.responseText); // Get 'timestamp'...
                    /* $ -> € and € -> $ only... */
                    if (this._from.common_symbol === '$')
                        this._exchange_rate.next(result.rates[`${this._to.iso_symbol}`]); // Based on RxJS 'Subject'...
                    else
                        this._exchange_rate.next(1 / result.rates[`${this._from.iso_symbol}`]); // Based on RxJS 'Subject'...
                }
            }
        };
    }

// Must pay:
//     public convert(amount: number, from_currency: Currency, to_currency: Currency): void {
//         window.console.assert(amount > 0, "'amount > 0', untrue?");
//         /* Calling 'open' for an already active request is the equivalent of calling 'abort' */
//         this._request.open('GET', 'http://openexchangerates.org/api/convert' + '/' + amount.toString() + '/' + from_currency.iso_symbol + '/' + to_currency.iso_symbol + '?app_id=' + '678cd96edd4b4f3eb637bf74ef8e0815', true);
//         // 'this._request' must already be opened:
//         this._request.send(null);
//     }

// Must pay:
//     public exchange_rate(from: Currency, to: Currency): void {
//         this._request.open('GET', 'http://openexchangerates.org/api/latest.json' + '?app_id=' + '678cd96edd4b4f3eb637bf74ef8e0815' + '&base=' + from.iso_symbol, true);
//         this._request.send(null);
//     }

    public exchange_rate_(from: Currency, to: Currency): void { // Free!
        this._from = from;
        this._to = to;
        // Stupid, request can be run once and for all in the constructor:
        this._request.open('GET', 'http://openexchangerates.org/api/latest.json' + '?app_id=' + '678cd96edd4b4f3eb637bf74ef8e0815', true);
        this._request.send(null);
    }
}
