// 'currencies.component.ts' file
// tslint:disable
import {Component, EventEmitter, OnChanges, OnDestroy, OnInit, Input, Output, SimpleChanges} from '@angular/core';
// JavaScript import of service:
import {Currencies_service} from './currencies-service';

export interface Currency { // Exemple du dollar américain :
    common_symbol: string; // $
    common_name: string; // Dollar
    description?: string;
    iso_code: number; // 840
    iso_symbol: string;  // USD
    material_icon?: string;
    substitute?: Currency; // 'undefined'
    substitution_date: number | null;
}

@Component({
    // providers: [My_stateful_service], // Scope is component only...
    selector: 'currencies',
    templateUrl: './currencies.component.html',
    styleUrls: ['./currencies.component.css']
})
export class Currencies implements OnChanges, OnDestroy, OnInit {

// Note that "Dependency Injection" sets '_currencies_service' to the *SINGLETON INSTANCE* of 'Currencies_service':
    constructor(private readonly _currencies_service: Currencies_service) {
        // '_currencies_service' has not to be used there ('ngOnInit' instead)...
    }

    static readonly Dollar = 0;
    static readonly Euro = 1;

    public static readonly Currencies: Array<Currency> = [
        {
            common_symbol: "$",
            common_name: "Dollar",
            description: "First leading currency in the world...",
            iso_code: 840,
            iso_symbol: "USD",
            material_icon: 'attach_money',
            substitution_date: null
        },
        {
            common_symbol: "€",
            common_name: "Euro",
            description: "Second leading currency in the world...",
            iso_code: 978,
            iso_symbol: "EUR",
            material_icon: 'euro_symbol',
            substitution_date: null
        },
        {
            common_symbol: "Ls",
            common_name: "Lats",
            description: undefined,
            iso_code: 428,
            iso_symbol: "LVL",
            substitute: {
                common_symbol: "€",
                common_name: "Euro",
                description: "Second leading currency in the world...",
                iso_code: 978,
                iso_symbol: "EUR",
                material_icon: 'euro_symbol',
                substitution_date: null
            },
            substitution_date: Date.parse('01 Jan 2014 00:00:00 GMT')
        }
    ];
    // Angular requires instance fields for bindings (see 'currencies.component.html')
    public readonly currencies = Currencies.Currencies;

    public static Default_substitute(): Currency {
        return Currencies.Currencies[Currencies.Euro];
    }

    private _currency_of_interest: Currency | null;
    get currency_of_interest(): Currency | null {
        return this._currency_of_interest;
    }

    @Input() // Data binding from parent (currencies-wrapper) to child (currencies)...
    set currency_of_interest(currency_of_interest: Currency | null) /* : void */ { // TypeScript setter rejects returned type...
        this._currency_of_interest = currency_of_interest;
    }

    ngOnChanges(changes: SimpleChanges): void {
        if (changes.hasOwnProperty('currency_of_interest') && changes.currency_of_interest) {
            window.console.warn("'ngOnChanges': " + JSON.stringify(changes));
        }
    }

    ngOnDestroy(): void {
        window.console.warn("'Currencies' instance: 'ngOnDestroy'");
    }

    ngOnInit(): void {
        window.console.warn("'Currencies' instance: 'ngOnInit'");
        // 'this._currencies_service' may be used there whether data are requested at initialization time...
    }

    @Input()
    public checked(other_currency: Currency): boolean {
        return this._currency_of_interest === other_currency;
    }

// Must pay:
//     public convert_to(amount: number = 1, from_currency: Currency = Currencies._Currencies[0], to_currency: Currency = Currencies._Currencies[1]): void {
//         this._currencies_service.convert(amount, from_currency, to_currency);
//     }

    public exchange_rate(from: Currency = Currencies.Currencies[Currencies.Dollar], to: Currency = Currencies.Currencies[Currencies.Euro]): void {
        if (from === to) {
            window.alert('From ' + from.iso_symbol + ' To ' + to.iso_symbol + ' ' + 1.);
            return;
        }
        this._currencies_service.exchange_rate_(from, to);
        this._currencies_service.exchange_rate.subscribe(amount => { // Exchange rate is got in an asynchronous way, i.e., 'next'...
            window.alert('From ' + from.iso_symbol + ' To ' + to.iso_symbol + ' ' + amount);
        });
    }

    // Parent listens for child event (https://angular.io/guide/component-interaction#parent-listens-for-child-event)
    @Output() // Mandatory!
    private readonly _send_currency = new EventEmitter<Currency>();

    public send(currency: Currency): void {
        // window.alert('emit: ' + currency.common_symbol);
        this._send_currency.emit(currency);
    }
}
